#include <WiFi.h>
#include <PubSubClient.h>

// SETUP WIFI CREDENTIALS
const char* ssid = "YOUR_SSID";
const char* password = "YOUR_PWD";

// SETUP MQTT DETAILS
const char* mqtt_server = "192.168.1.63";
const char* mqtt_user = "YOUR_USER";
const char* mqtt_pass = "YOUR_OTHER_PWD";

WiFiClient espClient;
PubSubClient client(espClient);

const int ledPin = 13;
const int soilMoisturePin = 36;
const int pumpPin = 16;

void setup() {
  pinMode(ledPin, OUTPUT);
  Serial.begin(9600);
  setup_wifi();
  client.setServer(mqtt_server, 1883);
  client.setCallback(callback);

  pinMode(ledPin, INPUT); // don't really need to do that as it's ADC
  
}

void loop() {

  if (!client.connected()) reconnect();
  client.loop();

  // Publish every minute
  if (millis() % 60000 == 0) {

    // READ YOUR SENSOR DATA HERE
    float value = analogRead(soilMoisturePin);

    // Send value as characters
    char msg[50];
    snprintf (msg, 50, "%f", value);
    Serial.print("Publish message: ");
    Serial.println(msg);

    // SET THE TOPIC TO PUBLISH HERE
    client.publish("soil-moisture", msg);
  }
}

void callback(char* topic, byte* payload, unsigned int length) {
  
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.print("] ");
  for (int i = 0; i < length; i++) {
    Serial.print((char)payload[i]);
  }
  Serial.println();

  String strPayload = String((char*)payload);

  // If isWatering is true
  if (strPayload == true) {
    // turn pump on for 5 secs
    if (millis() % 5000 == 0) {
      digitalWrite(pumpPin, HIGH);
    }
  }
}

void reconnect() {
  
  // Loop until we're reconnected
  while (!client.connected()) {
    
    Serial.print("Attempting MQTT connection...");
    // Create a random client ID
    String clientId = "ESP32-Desk";
    clientId += String(random(0xffff), HEX);
   
    // Attempt to connect
    if (client.connect(clientId.c_str(), mqtt_user, mqtt_pass)) {
      Serial.println("connected");
      
      // SET THE TOPIC TO SUBSCRIBE HERE
      client.subscribe("isWatering");
    
    } else {

      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);

    }
  }
}

void setup_wifi() {
  
  delay(10);
  // We start by connecting to a WiFi network
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}
